package example;

import de.asap.pak.core.commandpool.spi.ICommandPool;
import de.asap.pak.core.context.api.IContext;
import de.asap.pak.core.context.impl.ContextBuilder;
import de.asap.pak.core.context.services.spi.IPersistenceService;
import de.asap.pak.core.engine.api.IEngine;
import de.asap.pak.core.engine.impl.EngineBuilder;
import de.asap.pak.core.engine.spi.mapping.IMappingService;
import de.asap.pak.core.model.api.IModel;
import de.asap.pak.core.simple.context.SimplePersistenceService;
import de.asap.pak.core.simple.context.SimpleServiceProviderFactory;
import de.asap.pak.extra.jarpool.JarCommandPool;
import de.asap.pak.modelinterpreter.bpmn.BPMNModelInterpreter;
import de.asap.pak.modelinterpreter.bpmn.mapping.MappingService;
import de.asap.pak.modelinterpreter.bpmn.model.validator.ModelException;

import java.io.InputStream;

/**
 * Class for creating an engine
 */
public final class EngineCreator {

	private EngineCreator() {
		//hidden
	}

	/**
	 * This method configures an engine using the PAK Engine builder by using default implementations
	 *
	 * @param is Input-Stream object, which refers to a BPMN-File
	 */
	public static IEngine createEngine(final InputStream is) throws ModelException {
		//in our case we want to execute a Bpmn therefore we create the interpreter to parse and create a model
		final BPMNModelInterpreter interpreter = BPMNModelInterpreter.withInputStreamSource(is);
		//*1 Creating an instance of Model which is going to be executed
		final IModel model = interpreter.parseToModel();
		//*2 In this example we use the JarCommandPool for providing the commands  to the app
		final ICommandPool pool = new JarCommandPool();
		//*3 Each Engine needs a "Context" which provides all needed data and services to the engine
		final ContextBuilder cb = new ContextBuilder();
		//*4 in order to create a context, a serviceprovider factory needs to be passed. The factory will create a ServiceProvider which will hold all important services for the engine
		final SimpleServiceProviderFactory factory = new SimpleServiceProviderFactory();
		//*6 the persistence service will save all data which is created when executing the workflow model
		factory.addService(IPersistenceService.class,new SimplePersistenceService());
		factory.addService(IMappingService.class, new MappingService());
		cb.setServiceProviderFactory(factory);
		//Create the Context instance
		final IContext context = cb.build();
		//*6 Now we got everything we need to build a engine
		final IEngine engine = new EngineBuilder().setCommandPool(pool).setContext(context).setModel(model).build();

		//we also register our EngineObserver we have written before
		final EngineObserver engineObserver = new EngineObserver();
		engine.registerEngineObserver(engineObserver);
		return engine;
	}
}
